<?php

class URLRouter {
	
	/**
	 * 
	 * @param string $url
	 */
	public static function routeRequest($url=null) {
		if (is_null($url)) {
			return self::processRoute( self::getHttpRoot() );
		} else {
			return self::processRoute( $url );
		}
	}
	
	/**
	 *
	 * @param string $slug_
	 * @return array|false
	 */
	protected static function getHubBySlug($slug_) {
		$slug = strtolower($slug_);
		foreach(getHubs() as $hub) {
			if (in_array($slug, $hub[6], true)) {
				return $hub;
			}
		}
		return false;
	}
	
	/**
	 * 
	 * @param string $ROUTE
	 */
	protected static function processRoute($ROUTE) {
		$matches = [];
		$arc = new ArchiveState();
			
		if (hasGet('y') || hasGet('q') || hasGet('h')) {
			self::processRouteLegacy();
		
		} elseif (preg_match('~^/download$~', $ROUTE, $matches)) {
			ArchiveStreamer::streamArchive();
			die();
			
		} elseif ($ROUTE == '/') {
			$HUBS = getHubs();
			redirect('/'.urlencode($HUBS[0][6][ count($HUBS[0][6])-1 ]).'/'.$HUBS[0][4].'/'.$HUBS[0][5]);
			
		} elseif (preg_match('~^/([^/]+)/(\d+)/(\d+)$~', $ROUTE, $matches)) {
			$hub = self::getHubBySlug($matches[1]);
			if (is_array($hub)) {
				$arc->setHub( $matches[1], $hub );
				$arc->year  = $matches[2];
				$arc->month = $matches[3];
				$arc->renderView();
			} else {
				$arc->renderError('Invalid request.');
			}
			
		} elseif (
			preg_match('~^/([^/]+)/(\d+)/(\d+)/(\d+)$~', $ROUTE, $matches) ||
			preg_match('~^/([^/]+)/(\d+)/(\d+)/page-(\d+)$~', $ROUTE, $matches)
		) {
			$hub = self::getHubBySlug($matches[1]);
			if (is_array($hub)) {
				$arc->setHub( $matches[1], $hub );
				$arc->page  = $matches[4];
				$arc->year  = $matches[2];
				$arc->month = $matches[3];
				$arc->renderView();
			} else {
				$arc->renderError('Invalid request.');
			}
				
		} elseif (preg_match('~^/([^/]+)/search/(.*)$~', $ROUTE, $matches)) {
			$hub = self::getHubBySlug($matches[1]);
			if (is_array($hub)) {
				$arc->setHub( $matches[1], $hub );
				$arc->query = rawurldecode($matches[2]);
				$arc->regex = false;
				$arc->renderSearch();
			} else {
				$arc->renderError('Invalid request.');
			}
			
		} elseif (preg_match('~^/([^/]+)/rx/(.*)~', $ROUTE, $matches)) {
			$hub = self::getHubBySlug($matches[1]);
			if (is_array($hub)) {
				$arc->setHub( $matches[1], $hub );
				$arc->query = rawurldecode($matches[2]);
				$arc->regex = true;
				$arc->renderSearch();
			} else {
				$arc->renderError('Invalid request.');
			}
			
		} else {
			$arc->renderError('Unknown route');
		}
		
	}
	
	/**
	 * 
	 */
	protected static function processRouteLegacy() {
		
		$hubid = hasGet('h') ? $_GET['h'] : 0;
		if (hasGet('q')) {
			if (hasGet('rx')) {
				redirect('/'.rawurlencode($hubid).'/rx/'.rawurlencode($_GET['q']));
			} else {
				redirect('/'.rawurlencode($hubid).'/search/'.rawurlencode($_GET['q']));
			}
			
		} else {
			if (hasGet('y') && hasGet('m')) {
				$year  = hasGet('y') ? intval($_GET['y']) : date('Y');
				$month = hasGet('m') ? intval($_GET['m']) : date('m');
				if (hasGet('p')) {
					$page = intval($_GET['p']);
					redirect('/'.rawurlencode($hubid).'/'.$year.'/'.$month.'/page-'.$page);
				} else {
					redirect('/'.rawurlencode($hubid).'/'.$year.'/'.$month);
				}
				
			} else {
				$HUBS = getHubs();
				if (hasGet('h') && is_array($hub = self::getHubBySlug($_GET['h']))) {
					redirect('/'.rawurlencode($_GET['h']).'/'.$hub[4].'/'.$hub[5]);
				} else {
					redirect('/');
				}
			}
		}
	}
		
	/**
	 * 
	 * @return type
	 */
	public static function getHttpRoot() {
		$base = $_SERVER['REQUEST_URI'];
		if ($base[0] !== '/') {
			$base = '/'.$base;
		}
		
		$qp = strpos($base, '?');
		if ($qp !== false) {
			$base = substr($base, 0, $qp);
		}
		
		return $base;
	}
	
}
